@@ -17,11 +17,11 @@ Geo::Coder::Free - Provides a Geo-Coding functionality using free databases
 
 =head1 VERSION
 
-Version 0.19
+Version 0.20
 
 =cut
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 our $alternatives;
 
@@ -229,6 +229,7 @@ sub reverse_geocode {
 	if($self->{'openaddr'}) {
 		if(wantarray) {
 			my @rc = $self->{'openaddr'}->geocode(\%param);
+			return @rc;
 		} elsif(my $rc = $self->{'openaddr'}->geocode(\%param)) {
 			return $rc;
 		}
@@ -284,6 +285,56 @@ sub run {
 	print Data::Dumper->new([\@rc])->Dump();
 }
 
+sub _normalize {
+	my $type = uc(shift);
+
+	$type = uc($type);
+
+	if(($type eq 'AVENUE') || ($type eq 'AVE')) {
+		return 'AVE';
+	} elsif(($type eq 'STREET') || ($type eq 'ST')) {
+		return 'ST';
+	} elsif(($type eq 'ROAD') || ($type eq 'RD')) {
+		return 'RD';
+	} elsif(($type eq 'COURT') || ($type eq 'CT')) {
+		return 'CT';
+	} elsif(($type eq 'CIR') || ($type eq 'CIRCLE')) {
+		return 'CIR';
+	} elsif(($type eq 'FT') || ($type eq 'FORT')) {
+		return 'FT';
+	} elsif(($type eq 'CTR') || ($type eq 'CENTER')) {
+		return 'CTR';
+	} elsif(($type eq 'PARKWAY') || ($type eq 'PKWY')) {
+		return 'PKWY';
+	} elsif($type eq 'BLVD') {
+		return 'BLVD';
+	} elsif($type eq 'PIKE') {
+		return 'PIKE';
+	} elsif(($type eq 'DRIVE') || ($type eq 'DR')) {
+		return 'DR';
+	} elsif(($type eq 'SPRING') || ($type eq 'SPG')) {
+		return 'SPRING';
+	} elsif(($type eq 'RDG') || ($type eq 'RIDGE')) {
+		return 'RDG';
+	} elsif(($type eq 'CRK') || ($type eq 'CREEK')) {
+		return 'CRK';
+	} elsif(($type eq 'LANE') || ($type eq 'LN')) {
+		return 'LN';
+	} elsif(($type eq 'PLACE') || ($type eq 'PL')) {
+		return 'PL';
+	} elsif(($type eq 'GRDNS') || ($type eq 'GARDENS')) {
+		return 'GRDNS';
+	} elsif(($type eq 'HWY') || ($type eq 'HIGHWAY')) {
+		return 'HWY';
+	}
+
+	# Most likely failure of Geo::StreetAddress::US, but warn anyway, just in case
+	if($ENV{AUTHOR_TESTING}) {
+		# warn $self->{'location'}, ": add type $type";
+		warn "Add type $type";
+	}
+}
+
 =head1 AUTHOR
 
 Nigel Horne <njh@bandsman.co.uk>
